@@ -215,6 +215,109 @@ where
     })
 }
 
+/// MUST get an EntryHashed at a given EntryHash.
+///
+/// The EntryHashed is NOT guaranteed to be associated with a valid (or even validated) Header/Element.
+/// For example, an invalid Element could be published and `must_get_entry` would return the EntryHashed.
+///
+/// This may be useful during validation callbacks where the validity and relevance of some content can be
+/// asserted by the CURRENT validation callback independent of an Element. This behaviour avoids the potential for
+/// eclipse attacks to lie about the validity of some data and cause problems for a hApp.
+/// If you NEED to know that a dependency is valid in order for the current validation logic
+/// (e.g. inductive validation of a tree) then `must_get_valid_element` is likely what you need.
+///
+/// `must_get_entry` is available in contexts such as validation where both determinism and network access is desirable.
+///
+/// An EntryHashed will NOT be returned if:
+/// - @TODO It is PURGED (community redacted entry)
+/// - @TODO ALL headers pointing to it are WITHDRAWN by the authors
+/// - ALL headers pointing to it are ABANDONED by ALL authorities due to validation failure
+/// - Nobody knows about it on the currently visible network
+///
+/// If an EntryHashed fails to be returned:
+///
+/// - Callbacks will return early with `UnresolvedDependencies`
+/// - Zome calls will receive a `WasmError` from the host
+pub fn must_get_entry(entry_hash: EntryHash) -> ExternResult<EntryHashed> {
+    HDK.with(|h| {
+        h.borrow()
+            .must_get_entry(MustGetEntryInput::new(entry_hash))
+    })
+}
+
+/// MUST get a SignedHeaderHashed at a given HeaderHash.
+///
+/// The SignedHeaderHashed is NOT guaranteed to be a valid (or even validated) Element.
+/// For example, an invalid Header could be published and `must_get_header` would return the `SignedHeaderHashed`.
+///
+/// This may be useful during validation callbacks where the validity depends on a Header existing regardless of its associated Entry.
+/// For example, we may simply need to check that the author is the same for two referenced Headers.
+///
+/// `must_get_header` is available in contexts such as validation where both determinism and network access is desirable.
+///
+/// A `SignedHeaderHashed` will NOT be returned if:
+///
+/// - @TODO The header is WITHDRAWN by the author
+/// - @TODO The header is ABANDONED by ALL authorities
+/// - Nobody knows about it on the currently visible network
+///
+/// If a `SignedHeaderHashed` fails to be returned:
+///
+/// - Callbacks will return early with `UnresolvedDependencies`
+/// - Zome calls will receive a `WasmError` from the host
+pub fn must_get_header(header_hash: HeaderHash) -> ExternResult<SignedHeaderHashed> {
+    HDK.with(|h| {
+        h.borrow()
+            .must_get_header(MustGetHeaderInput::new(header_hash))
+    })
+}
+
+/// MUST get a VALID Element at a given HeaderHash
+///
+/// The Element is guaranteed to be valid.
+/// More accurately the Element is guarantee to be consistently reported as valid by the visible network.
+///
+/// The validity requirement makes this more complex but notably enables inductive validation of arbitrary graph structures.
+/// For example "If this Element is valid, and its parent is valid, up to the root, then the whole tree of Elements is valid".
+///
+/// If at least one authority (1 of N trust) claims the Element is invalid then a conflict resolution/warranting round will be triggered.
+///
+/// In the case of a total eclipse (every visible authority is lying) then we cannot immediately detect an invalid Element.
+/// Unlike `must_get_entry` and `must_get_header` we cannot simply inspect the cryptographic integrity to know this.
+///
+/// In theory we can run validation of the returned Element ourselves, which itself may be based on `must_get_X` calls.
+/// If there is a large nested graph of `must_get_valid_element` calls this could be extremely heavy.
+/// Note though that each "hop" in recursive validation is routed to a completely different set of authorities.
+/// It does not take many hops to reach the point where an attacker needs to eclipse the entire network to lie about Element validity.
+///
+/// @TODO We keep signed receipts from authorities serving up "valid elements".
+/// - If we ever discover an element we were told is valid is invalid we can retroactively look to warrant authorities
+/// - We can async (e.g. in a background task) be recursively validating Element dependencies ourselves, following hops until there is no room for lies
+/// - We can with small probability recursively validate to several hops inline to discourage potential eclipse attacks with a credible immediate threat
+///
+/// If you do not care about validity and simply want a pair of Header+Entry data, then use both `must_get_header` and `must_get_entry` together.
+///
+/// `must_get_valid_element` is available in contexts such as validation where both determinism and network access is desirable.
+///
+/// An `Element` will not be returned if:
+///
+/// - @TODO It is WITHDRAWN by the author
+/// - @TODO The Entry is PURGED by the community
+/// - It is ABANDONED by ALL authorities due to failed validation
+/// - If ANY authority (1 of N trust) OR ourselves (0 of N trust) believes it INVALID
+/// - Nobody knows about it on the visible network
+///
+/// If an `Element` fails to be returned:
+///
+/// - Callbacks will return early with `UnresolvedDependencies`
+/// - Zome calls will receive a `WasmError` from the host
+pub fn must_get_valid_element(header_hash: HeaderHash) -> ExternResult<Element> {
+    HDK.with(|h| {
+        h.borrow()
+            .must_get_valid_element(MustGetValidElementInput::new(header_hash))
+    })
+}
+
 /// Get an element from the hash AND the details for the entry or header hash passed in.
 /// Returns [ `None` ] if the entry/header does not exist.
 /// The details returned are a contextual mix of elements and header hashes, see below.
@@ -313,6 +416,30 @@ macro_rules! app_entry {
             }
         }
 
+        impl TryFrom<$crate::prelude::EntryHashed> for $t {
+            type Error = $crate::prelude::WasmError;
+            fn try_from(entry_hashed: $crate::prelude::EntryHashed) -> Result<Self, Self::Error> {
+                Self::try_from(entry_hashed.as_content())
+            }
+        }
+
+        impl TryFrom<&$crate::prelude::Element> for $t {
+            type Error = $crate::prelude::WasmError;
+            fn try_from(element: &$crate::prelude::Element) -> Result<Self, Self::Error> {
+                Ok(match element.entry() {
+                    ElementEntry::Present(serialized) => Self::try_from(serialized)?,
+                    _ => return Err(Self::Error::Guest(format!("Tried to deserialize an element, expecting it to contain entry data, but there was none. Element HeaderHash: {}", element.header_hashed().as_hash()))),
+                })
+            }
+        }
+
+        impl TryFrom<$crate::prelude::Element> for $t {
+            type Error = $crate::prelude::WasmError;
+            fn try_from(element: $crate::prelude::Element) -> Result<Self, Self::Error> {
+                (&element).try_into()
+            }
+        }
+
         impl TryFrom<&$t> for $crate::prelude::Entry {
             type Error = $crate::prelude::WasmError;
             fn try_from(t: &$t) -> Result<Self, Self::Error> {
@@ -586,12 +713,6 @@ macro_rules! entry_def_index {
                     }
                 }
             }
-            Ok($crate::prelude::EntryDefsCallbackResult::Err(error)) => {
-                $crate::prelude::tracing::error!(?error, "Failed to lookup entry defs.");
-                Err::<$crate::prelude::EntryDefIndex, $crate::prelude::WasmError>(
-                    $crate::prelude::WasmError::Guest(error),
-                )
-            }
             Err(error) => {
                 $crate::prelude::tracing::error!(?error, "Failed to lookup entry defs.");
                 Err::<$crate::prelude::EntryDefIndex, $crate::prelude::WasmError>(error)
